﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace CsCgrep
{
    /// <summary>
    /// 結果出力ダイアログ
    /// </summary>
    public partial class OutputDialog : Form
    {
        /// <summary>
        /// 親ウィンドウ
        /// </summary>
        private MainWindow parent;

        /// <summary>
        /// 結果出力ダイアログを作成します。
        /// </summary>
        /// <param name="parent">親ウィンドウ</param>
        public OutputDialog(MainWindow parent)
        {
            InitializeComponent();

            this.parent = parent;

            // ダイアログ入力情報の復元
            RestoreDialogSettings();
        }

        /// <summary>
        /// ダイアログ入力情報を設定ファイルから復元します。
        /// </summary>
        private void RestoreDialogSettings()
        {
            // ダイアログ入力情報の復元
            fileNameCheckBox.Checked = parent.AppSettings.FileNameChecked;
            placeCheckBox.Checked = parent.AppSettings.PlaceChecked;
            lineNumberCheckBox.Checked = parent.AppSettings.LineNumberChecked;
            contentsCheckBox.Checked = parent.AppSettings.ContentsChecked;
            markerCheckBox.Checked = parent.AppSettings.MarkerChecked;

            spaceRadioButton.Checked = parent.AppSettings.SpaceRadioChecked;
            tabRadioButton.Checked = parent.AppSettings.TabRadioChecked;
            commaRadioButton.Checked = parent.AppSettings.CommaRadioChecked;
            anyRadioButton.Checked = parent.AppSettings.AnyRadioChecked;

            markerTextBox.Text = parent.AppSettings.MarkerText;
            spaceNumBox.Text = parent.AppSettings.SpaceNum;
            tabNumBox.Text = parent.AppSettings.TabNum;
            anyTextBox.Text = parent.AppSettings.AnyText;
        }

        /// <summary>
        /// ダイアログ入力情報を保存します。
        /// </summary>
        private void SaveDialogSettings()
        {
            parent.AppSettings.FileNameChecked = fileNameCheckBox.Checked;
            parent.AppSettings.PlaceChecked = placeCheckBox.Checked;
            parent.AppSettings.LineNumberChecked = lineNumberCheckBox.Checked;
            parent.AppSettings.ContentsChecked = contentsCheckBox.Checked;
            parent.AppSettings.MarkerChecked = markerCheckBox.Checked;

            parent.AppSettings.SpaceRadioChecked = spaceRadioButton.Checked;
            parent.AppSettings.TabRadioChecked = tabRadioButton.Checked;
            parent.AppSettings.CommaRadioChecked = commaRadioButton.Checked;
            parent.AppSettings.AnyRadioChecked = anyRadioButton.Checked;

            parent.AppSettings.MarkerText = markerTextBox.Text;
            parent.AppSettings.SpaceNum = spaceNumBox.Text;
            parent.AppSettings.TabNum = tabNumBox.Text;
            parent.AppSettings.AnyText = anyTextBox.Text;
        }

        /// <summary>
        /// 検索結果をファイルへ出力ボタンクリック時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void outputButton_Click(object sender, EventArgs e)
        {
            // 出力項目が1つも選択されていない場合はエラー
            if (!fileNameCheckBox.Checked &&
                !placeCheckBox.Checked &&
                !lineNumberCheckBox.Checked &&
                !contentsCheckBox.Checked)
            {
                MessageUtil.ShowAlert("出力項目を選択してください。");
                return;
            }

            if (spaceRadioButton.Checked)
            {
                try
                {
                    Int32.Parse(spaceNumBox.Text);
                }
                catch (FormatException)
                {
                    MessageUtil.ShowAlert("スペースの個数は数値で入力してください。");
                    return;
                }
            }

            if (tabRadioButton.Checked)
            {
                try
                {
                    Int32.Parse(tabNumBox.Text);
                }
                catch (FormatException)
                {
                    MessageUtil.ShowAlert("タブの個数は数値で入力してください。");
                    return;
                }
            }

            if (anyRadioButton.Checked)
            {
                if (anyTextBox.Text.Trim() == "")
                {
                    MessageUtil.ShowAlert("任意文字列を入力したください。");
                    return;
                }
            }

            // SaveFileDialog の新しいインスタンスを生成する (デザイナから追加している場合は必要ない)
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // ダイアログのタイトルを設定する
            saveFileDialog1.Title = "名前を付けて保存";

            // 初期表示するディレクトリを設定する
            saveFileDialog1.InitialDirectory = @"C:\";

            // 初期表示するファイル名を設定する
            saveFileDialog1.FileName = "";

            // ファイルのフィルタを設定する
            saveFileDialog1.Filter = "テキスト ファイル|*.txt;*.log|すべてのファイル|*.*";

            // ファイルの種類 の初期設定を 1 番目に設定する (初期値 1)
            saveFileDialog1.FilterIndex = 1;

            // ダイアログを表示し、戻り値が [OK] の場合は、選択したファイルを表示する
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                DoOutput(saveFileDialog1.FileName);

                // 不要になった時点で破棄する (正しくは オブジェクトの破棄を保証する を参照)
                saveFileDialog1.Dispose();

                MessageUtil.ShowInfo("検索結果をファイルへ出力しました。");

                // ダイアログの入力情報を保存
                SaveDialogSettings();

                Close();
            }
            else
            {
                // 不要になった時点で破棄する (正しくは オブジェクトの破棄を保証する を参照)
                saveFileDialog1.Dispose();
                return;
            }

        }

        /// <summary>
        /// 検索結果をファイルへ出力します。
        /// </summary>
        /// <param name="fileName">出力ファイル名</param>
        private void DoOutput(string fileName)
        {
            StringBuilder header = new StringBuilder();

            if (markerCheckBox.Checked)
            {
                Encoding sjisEnc = Encoding.GetEncoding(Constants.SHIFT_JIS);

                int length = sjisEnc.GetByteCount(markerTextBox.Text);

                for (int i = 0; i < (length + 1); i++)
                {
                    header.Append(" ");
                }
            }

            StringBuilder separator = new StringBuilder();

            if (spaceRadioButton.Checked)
            {
                for (int i = 0; i < Int32.Parse(spaceNumBox.Text); i++)
                {
                    separator.Append(" ");
                }
            }
            else if (tabRadioButton.Checked)
            {
                for (int i = 0; i < Int32.Parse(tabNumBox.Text); i++)
                {
                    separator.Append("\t");
                }
            }
            else if (commaRadioButton.Checked)
            {
                separator.Append(",");
            }
            else if (anyRadioButton.Checked)
            {
                separator.Append(anyTextBox.Text);
            }


            StreamWriter writer = new StreamWriter(fileName, false, Encoding.GetEncoding(parent.AppSettings.OutputEncoding));

            try
            {
                foreach (SearchResult r in parent.SearchResults)
                {
                    // ブロック間の区切り行の場合
                    if (r.RowNumber == 0)
                    {
                        writer.WriteLine(); // 改行を出力
                        continue;
                    }

                    if (markerCheckBox.Checked)
                    {
                        if (r.IsMatchLine)
                        {
                            writer.Write(markerTextBox.Text + " ");
                        }
                        else
                        {
                            writer.Write(header.ToString());
                        }
                    }

                    if (fileNameCheckBox.Checked)
                    {
                        writer.Write(r.FileName);

                        if (placeCheckBox.Checked || lineNumberCheckBox.Checked || contentsCheckBox.Checked)
                        {
                            writer.Write(separator.ToString());
                        }
                        else
                        {
                            writer.WriteLine();
                        }
                    }

                    if (placeCheckBox.Checked)
                    {
                        writer.Write(r.Place);

                        if (lineNumberCheckBox.Checked || contentsCheckBox.Checked)
                        {
                            writer.Write(separator.ToString());
                        }
                        else
                        {
                            writer.WriteLine();
                        }
                    }

                    if (lineNumberCheckBox.Checked)
                    {
                        if (r.RowNumber > 0)
                        {
                            writer.Write(r.RowNumber.ToString());
                        }
                        else
                        {
                            writer.Write("");
                        }

                        if (contentsCheckBox.Checked)
                        {
                            writer.Write(separator.ToString());
                        }
                        else
                        {
                            writer.WriteLine();
                        }
                    }

                    if (contentsCheckBox.Checked)
                    {
                        writer.WriteLine(r.Contents);
                    }
                }
            }
            catch (IOException)
            {
                MessageUtil.ShowAlert("ファイルの書き込み中にエラーが発生しました。");
                return;
            }
            finally
            {
                if (writer != null)
                {
                    writer.Close();
                }
            }
        }

        /// <summary>
        /// キャンセルボタンクリック時の処理です。
        /// </summary>
        /// <param name="sender">イベントを発生させたオブジェクト</param>
        /// <param name="e">イベント</param>
        private void cancelButton_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}
